
/**
* 
* @module Kiwi
* @submodule Plugins
*
*/

//Make sure to assign a value to 'Kiwi.Plugins.FlurryAnalytics.API_KEY'


/**
* 
* 
* @namespace Kiwi.Plugins
* @class GamefrootFlurryAnalytics
* @static
* 
*/
Kiwi.Plugins.GamefrootFlurryAnalytics = {

	/**
	*
	* @property name
	* @type String
	* @static
	* @default 'GamefrootFlurryAnalytics'
	* @public
	*/
	name: 'GamefrootFlurryAnalytics',

	/**
	*
	* @property version
	* @type String
	* @default '0.5.0'
	* @static
	* @readOnly
	* @public
	*/
	version: '0.5.0',

	/**
	* 
	* @property minimumKiwiVersion
	* @type String
	* @default '1.3.0'
	* @static
	* @readOnly
	* @public
	*/
	minimumKiwiVersion: '1.3.0',


	/**
	* 
	* @property pluginDependencies
	* @type Array
	* @default []
	* @static
	* @readOnly
	* @public
	*/
	pluginDependencies:  [
		{
			name: "FlurryAnalytics",
			minimumVersion: "0.1.0"
		}
	],

	/**
	*
	* @property kiwifrootPlugins
	* @type Array
	* @public
	*/
	kiwifrootPlugins: [
	]
	

};	

// Register this with the kiwi plugin manager
Kiwi.PluginManager.register( Kiwi.Plugins.GamefrootFlurryAnalytics );


Kiwi.Plugins.GamefrootFlurryAnalytics.create = function( game ) {

	var i = 0,
		state;
	while( i < game.states._states.length ) {

		state = game.states._states[ i ];

		if( state.onExecute ) {
			state.onExecute.add( Kiwi.Plugins.GamefrootFlurryAnalytics.onExecute, game );
		}

		i++;
	}

};

Kiwi.Plugins.GamefrootFlurryAnalytics.onExecute = function(event, info) {

	switch( event ) {

	//Launch State
		//Unavaliable events due to the current set-up of Kiwifroot
		case Kiwifroot.States.Launch.EVENTS.PRELOAD: 
		case Kiwifroot.States.Launch.EVENTS.CREATE:
		case Kiwifroot.States.Launch.EVENTS.GLOBAL_ASSETS_LOAD:
		case Kiwifroot.States.Launch.EVENTS.GLOBAL_ASSETS_COMPLETE:
		case Kiwifroot.States.Launch.EVENTS.GLOBAL_ASSETS_RELOAD:
		case Kiwifroot.States.Launch.EVENTS.GLOBAL_ASSETS_FAILED:
			break;


		case Kiwifroot.States.Launch.EVENTS.ERROR:
			this.flurry.logError( 'Error:Launch', info );
			break;
		case Kiwifroot.States.Preloader.EVENTS.ERROR:
			this.flurry.logError( 'Error:Preloader', info );
			break;

	//Preloader State
		case Kiwifroot.States.Preloader.EVENTS.PRELOAD:
			this.flurry.logEvent( 'Preloader', { level: this.game.info.currentLevel.id, time: Date.now() }, true );
			this.flurry.logEvent( 'Preloader:Load Level JSON', { time: Date.now() }, true );
			break;
		case Kiwifroot.States.Preloader.EVENTS.CREATE:
			this.flurry.endTimedEvent( 'Preloader:Load Level JSON', { time: Date.now() } );
			break;
		case Kiwifroot.States.Preloader.EVENTS.ASSETS_LOAD:
			this.flurry.logEvent( 'Preloader:Load Level Assets', { time: Date.now() }, true );
			break;
		case Kiwifroot.States.Preloader.EVENTS.ASSETS_RELOAD:
			this.flurry.logEvent( 'Preloader:Reload Level Assets' );
			break;
		case Kiwifroot.States.Preloader.EVENTS.ASSETS_FAILED:
			this.flurry.logEvent( 'Preloader:Failed Loading Level Assets', { time: Date.now() } );
			break;
		case Kiwifroot.States.Preloader.EVENTS.ASSETS_COMPLETE:
			this.flurry.endTimedEvent( 'Preloader:Load Level Assets', { time: Date.now() } );
			break;
		case Kiwifroot.States.Preloader.EVENTS.SHUT_DOWN:
			this.flurry.endTimedEvent( 'Preloader' );
			break;

	//Level State
		case Kiwifroot.States.Level.EVENTS.PRELOAD:
			this.flurry.logEvent( 'Level', { level: this.game.info.currentLevel.id, time: Date.now() }, true );
			break;
		case Kiwifroot.States.Level.EVENTS.CREATE:
			this.flurry.logEvent( 'Level:Create' ); 
			break;
		case Kiwifroot.States.Level.EVENTS.START: 
			this.flurry.logEvent( 'Level:Start' );
			break;
		case Kiwifroot.States.Level.EVENTS.SHUT_DOWN:
			this.flurry.endTimedEvent( 'Level', { time: Date.now() } );
			break;
		case Kiwifroot.States.Level.EVENTS.PAUSED:
			this.flurry.logEvent('Level:Paused', { time: Date.now() } );
			break;
		case Kiwifroot.States.Level.EVENTS.UN_PAUSED:
			this.flurry.logEvent('Level:Unpaused', { time: Date.now() } );
			break;

	}

}