Kiwi.Plugins.Gamemanager = {
	name:'Gamemanager',
	version:'0.5.0',
	minimumKiwiVersion:'1.1.1',
	kiwifrootPlugins:[]
};

Kiwi.PluginManager.register(Kiwi.Plugins.Gamemanager);

Kiwi.Plugins.Gamemanager.Component = function(gameObject){
	Kiwi.Component.call(this, gameObject, 'Gamemanager');

	this.state = this.owner.state;
	this.box = this.owner.components.getComponent("Box");

	this.owner.properties.onUpdate( this.onMessageRecieved, this, "_messaging_");
	this.game.input.onUp.add(this.onStageTouch2, this, 25);
	this.state.robots.onUpdate.add(this.onUpdate, this);
};

Kiwi.extend(Kiwi.Plugins.Gamemanager.Component, Kiwi.Component);

Kiwi.Plugins.Gamemanager.Component.prototype.boot = function() {
	this.arcadephysics = this.owner.components.getComponent( "ArcadePhysics" );

	if( !this.arcadephysics ) {
		Kiwi.Log.error("ArcadePhysics component not found on the state. ArcadePhysics blocks will not work.", "#arcadephysics");
	} else {
		this.arcadesystem = this.arcadephysics.system;
	}

	//Attach the information to the boot method.
	//This is to make sure all the components are loaded/attached before executing any code.
	this.onCreate();
};

Kiwi.Plugins.Gamemanager.Component.add = function(state, gameObject, params){
	var component = new Kiwi.Plugins.Gamemanager.Component(gameObject);
	gameObject.components.add(component);

	component.pointer = params["pointer"];
	component.tile_instance = params["tile-instance"];
	component.tile_instances = params["tile-instances"];
	component.current_tiles = params["current-tiles"];
	component.highest_tile = params["highest-tile"];
	component.working_var_1 = params["working-var-1"];
	component.speed = params["speed"];
	component.game_started = params["game-started"];
	component.game_over = params["game-over"];
	component.tile = params["tile"];
	component.top_connector = params["top-connector"];
	component.index = params["index"];
	component.working_var_2 = params["working-var-2"];
	component.player = params["player"];
	component.i = params["i"];
	component.score = params["score"];
	component.startTime = params["startTime"];
};

Kiwi.Plugins.Gamemanager.Component.prototype.executeMessageRecieved = function() {
	if (this.game_started) {
		this.game_started = false;
		this.game_over = true;
		this.scoreText.visible = false;
		this.score = (this.game.time.clock.elapsed() * this.game.time.clock.units) - this.startTime;
		//now lets display the game over screen...
		
		var gameover = this.state.getFirstChildByTag("Gameoverscreen");

		if ( gameover ){
			gameover.visible = true;
			gameover.components.getComponent("Gameoverscreen").show( this.score );
			//now lets create a new text object...
		}
	}
};

Kiwi.Plugins.Gamemanager.Component.prototype.stopScrolling = function() {
	this.scrolling_disabled = true;
}

Kiwi.Plugins.Gamemanager.Component.prototype.onCreate = function() {
	this.createInitVariables();
	this.arcadephysics.immovable = true;
	this.arcadephysics.enabled = false;
	this.owner.addTag("manager");
	this.owner.x = this.game.stage.width - this.owner.width;

	this.scoreText = this.state.objects.create( {
		"type": 'text', 
		"x": this.game.stage.width - 10,
		"y": 15,
		"text": '000000',
		"textAlign": "right",
		"fontFamily": '"nokia"',
		"color": "#ffffff",
		"fontSize": 30
    }, this.state, false );
    this.scoreText.visible = false;
	this.owner.visible = false;
	
};

Kiwi.Plugins.Gamemanager.Component.prototype.executeMessageRecieved2 = function() {
	this.tile_instances.push( this.tile_instance );
};

Kiwi.Plugins.Gamemanager.Component.prototype.onUpdate = function() {
	if (this.game_started) {
		this.loopTiles();
		if( this.speed < 650000000 ) {
			this.speed = this.speed + this.speed * (1.1 * (1 / 60) );
			this.last_speed = 650000000;
		} else {
			this.speed = this.speed + this.speed * ( (this.last_speed / this.speed) * (1 / 60) );
			this.last_speed = this.speed;
		}
		this.updateScore();
	}

	this.scoreText.text = this.scoreText.text;
	
};

Kiwi.Plugins.Gamemanager.Component.prototype.updateScore = function() {
	var score = Math.round( ( this.game.time.clock.elapsed() * this.game.time.clock.units ) - this.startTime ).toString();
	while(score.length < 6) {
		score = '0' + score;
	}
	this.scoreText.text = score;
	this.state.addChild( this.scoreText );
};

Kiwi.Plugins.Gamemanager.Component.prototype.proceedToNextLevel = function() {
	this.game.info.switchLevelByNumber((this.game.info.currentLevelNumber + 1));
}

Kiwi.Plugins.Gamemanager.Component.prototype.onMessageRecieved = function( name, message ) {
	switch( message ) {
		case 'GAME-OVER': this.executeMessageRecieved(); break;
		case 'STOP': this.stopScrolling(); break;
		case 'TILE-INSTANCE':	
			this.tile_instance = this.owner.properties.get("_messaging-value_");
			this.executeMessageRecieved2();
			break;
		case 'slowDown': this.slowDown(); break;
		case 'COLLECTABLE-INSTANCE':
			var collectable = this.owner.properties.get("_messaging-value_");
			this.collectables.push( collectable );
			break;
	}
};

Kiwi.Plugins.Gamemanager.Component.prototype.slowDown = function(){
	var speed = this.speed;
	this.speed = ( speed / 2 );
	this.game.time.clock.setTimeout( function() {
		this.speed = speed;
	}, 10000, this);
} 

Kiwi.Plugins.Gamemanager.Component.prototype.onStageTouch2 = function( x, y, td, tu, dur, pointer ) {
	this.pointer = pointer;
	if (this.game_over) {
		//this.proceedToNextLevel();
	} else if (!this.game_started) {
		this.startGame();
	}

};

Kiwi.Plugins.Gamemanager.Component.prototype.createInitVariables = function() {
	this.tile_instances = [];
	this.current_tiles = [];
	this.collectables = [];
	this.player = this.state.getFirstChildByTag('player');
	this.game_started = false;
	this.speed = 20;
	this._result = new Kiwi.Geom.IntersectResult();
}

Kiwi.Plugins.Gamemanager.Component.prototype.messageEachGameObject = function( group, message ) {
	group.forEach( this, function( child ) {
		child.properties.set( "_messaging_", message );
		if( child.childType() == Kiwi.Group ) {
			this.messageEachGameObject( child, message );
		}
	});
	return;
}

Kiwi.Plugins.Gamemanager.Component.prototype.requestAllTileInstances = function() {
	this.tile_instances = [];
	this.messageEachGameObject( this.state, 'REQUEST-TILE-INSTANCES');
}

Kiwi.Plugins.Gamemanager.Component.prototype.spawnNewTile = function() {
	
	this.tile_instances.sort( function() { return (Math.random() - 0.5) * 10; } );
	for (var tile_index in this.tile_instances) {
		this.tile = this.tile_instances[tile_index];
		if (!this.tile.hasTag('current-tile')) {
			if (this.top_connector == 1 && this.tile.hasTag('bottom-connector-1')) {
				this._createNewTile();
				break;
			} else if (this.top_connector == 2 && this.tile.hasTag('bottom-connector-2')) {
				this._createNewTile();
				break;
			} else if (this.top_connector == 3 && this.tile.hasTag('bottom-connector-3')) {
				this._createNewTile();
				break;
			}
		}
	}
}

Kiwi.Plugins.Gamemanager.Component.prototype.startGame = function() {
	this.requestAllTileInstances();
	if (this.player == null) {
		this.player = this.state.getFirstChildByTag('player');
	}
	this.setCurrentTilesFromScreen();
	this.game_started = true;
	this.messageEachGameObject( this.state, 'START-GAME');
	this.startTime = (this.game.time.clock.elapsed() * this.game.time.clock.units);
	this.scoreText.visible = true;
	//this.owner.visible = true;
}

Kiwi.Plugins.Gamemanager.Component.prototype.setCurrentTilesFromScreen = function() {
	this.current_tiles = [];
	for (var tile_index2 in this.tile_instances) {
		this.tile = this.tile_instances[tile_index2];
		if (this.tile.x <= this.game.stage.width) {
			if (this.tile.y <= this.game.stage.height) {
				this.tile.addTag('current-tile');
				this.tile.visible = true;
				this.tile.x = 0;
				this.current_tiles.push( this.tile );
			}
		}
	}
}

Kiwi.Plugins.Gamemanager.Component.prototype.loopTiles = function() {
	this.highest_tile = 540;
	this.top_connector = 2;
	var i_start = this.current_tiles.length;
	var i_inc = 1;
	if (i_start > 1) {
		i_inc = -i_inc;
	}
	for (i = i_start; i_inc >= 0 ? i <= 1 : i >= 1; i += i_inc) {
		this.i = i;
		this.tile = this.current_tiles[this.i - 1];
		if (this.tile != null) {
			this.tileOverlappingPlayerCheck(this.tile);
			this.tilePositionCheck(this.tile);
		}
	}
	this.checkNewTileSpawn();
	this.updateCollectables();

	//Player offscreen check
	if( this.player.x + this.player.width < 0 || this.player.x > this.game.stage.width ) {
		this.player.properties.set("_messaging_", 'OFF-ROAD');
	}
}

Kiwi.Plugins.Gamemanager.Component.prototype._createNewTile = function() {
	this.tile.addTag('current-tile');
	this.tile.visible = true;
	this.tile.x = 0;
	this.tile.y = (this.highest_tile - this.tile.height) + 2;
	this.tile.properties.set("_messaging_", 'createCollectable');
	this.current_tiles.push( this.tile );
}

Kiwi.Plugins.Gamemanager.Component.prototype.checkNewTileSpawn = function() {
	if (this.highest_tile > -540) {
		this.spawnNewTile();
	}
}

Kiwi.Plugins.Gamemanager.Component.prototype.updateCollectables = function( collectable ) {

	var collectable;

	for( var i = 0; i < this.collectables.length; i++) {

		collectable = this.collectables[ i ];

		if( !this.scrolling_disabled ) {
			collectable.y = Math.round( collectable.y + Math.log(this.speed) / Math.log(10) );
		}

		if( this.game.stage.height <= collectable.y || this.collectableOverlapPlayerCheck( collectable ) ) {
			collectable.exists = false;
			this.collectables.splice( i, 1 );
			i--;
			continue;
		}

	}

}

Kiwi.Plugins.Gamemanager.Component.prototype.collectableOverlapPlayerCheck = function( collectable ) {
	
	if( Kiwi.Geom.Intersect.rectangleToRectangle( collectable.components.getComponent('ArcadePhysics').getHitbox(), 
		this.player.components.getComponent('ArcadePhysics').getHitbox() ).result ) {
		this.speed = this.speed * 0.2;
		return true;
	} else {
		return false;
	}

}

Kiwi.Plugins.Gamemanager.Component.prototype.tilePositionCheck = function(tile_instance) {
	this.tile_instance = tile_instance;
	if (this.game.stage.height <= this.tile_instance.y) {
		this.index = this.current_tiles.indexOf(this.current_tiles) + 1;
		this.tile_instance.visible = false;
		this.tile_instance.removeTag('current-tile');
		while (this.current_tiles.indexOf(this.tile_instance) + 1 != 0) {
			this.current_tiles.splice(this.current_tiles.indexOf(this.tile_instance) + 1 - 1, 1);
		}
	} else {	

		if( !this.scrolling_disabled ) {
			this.tile_instance.y = Math.round( this.tile_instance.y + Math.log(this.speed) / Math.log(10) );	
		}

		if (this.highest_tile > this.tile_instance.y) {
			this.highest_tile = this.tile_instance.y;
			if (this.tile.hasTag('top-connector-1')) {
				this.top_connector = 1;
			} else if (this.tile.hasTag('top-connector-3')) {
				this.top_connector = 3;
			} else {
				this.top_connector = 2;
			}
		}

		
	}
}

Kiwi.Plugins.Gamemanager.Component.prototype.tileOverlappingPlayerCheck = function(tile_instance) {
	this.tile_instance = tile_instance;
	this.working_var_1 = this.player.x + this.player.width / 2;
	this.working_var_2 = this.player.y + this.player.height / 2;
	if (this.working_var_1 >= this.tile_instance.x && this.working_var_1 <= this.tile_instance.x + this.tile_instance.width) {
		if (this.working_var_2 >= this.tile_instance.y && this.working_var_2 <= this.tile_instance.y + this.tile_instance.height) {
			this.tile_instance.properties.set("_messaging-value_", this.player);
			this.tile_instance.properties.set("_messaging_", 'CHECK-ON-ROAD');
		}
	}
}


Kiwi.Plugins.Gamemanager.Component.prototype.destroy = function(){
	this.game.input.onUp.remove(this.onStageTouch2, this);
	this.state.robots.onUpdate.remove(this.onUpdate, this);

	Kiwi.Component.prototype.destroy.call( this );
	this.state = null;
	this.box = null;
};

Kiwi.Plugins.Gamemanager.kiwifrootPlugins.push({
	type: Kiwifroot.Plugins.COMPONENT,
	name: 'Gamemanager',
	namespace: Kiwi.Plugins.Gamemanager.Component
});
