Kiwi.Plugins.Maptile = {
	name:'Maptile',
	version:'0.5.0',
	minimumKiwiVersion:'1.1.1',
	kiwifrootPlugins:[]
};

Kiwi.PluginManager.register(Kiwi.Plugins.Maptile);

Kiwi.Plugins.Maptile.Component = function(gameObject){
	Kiwi.Component.call(this, gameObject, 'Maptile');

	this.state = this.owner.state;
	this.box = this.owner.components.getComponent("Box");

	this.owner.properties.onUpdate( this.onMessageRecieved, this, "_messaging_");
};

Kiwi.extend(Kiwi.Plugins.Maptile.Component, Kiwi.Component);

Kiwi.Plugins.Maptile.Component.prototype.boot = function() {
	this.arcadephysics = this.owner.components.getComponent( "ArcadePhysics" );

	if( !this.arcadephysics ) {
		Kiwi.Log.error("ArcadePhysics component not found on the state. ArcadePhysics blocks will not work.", "#arcadephysics");
	} else {
		this.arcadesystem = this.arcadephysics.system;
	}

	//Attach the information to the boot method.
	//This is to make sure all the components are loaded/attached before executing any code.
	this.onCreate();
};

Kiwi.Plugins.Maptile.Component.add = function(state, gameObject, params){
	var component = new Kiwi.Plugins.Maptile.Component(gameObject);
	gameObject.components.add(component);

	component.road_markers_string = params["road-markers-string"];
	component.instance = params["instance"];
	component.useCenter = params["useCenter"];
	component.randomVar = params["randomVar"];
	component.road_markers = params["road-markers"];
	component.collectable = params["collectable"];
	component.i = params["i"];
	component.instance_x = params["instance-x"];
	component.lerp_y = params["lerp-y"];
	component.collectable_y = params["collectable-y"];
	component.j = params["j"];
	component.instance_y = params["instance-y"];
	component.prev_y = params["prev-y"];
	component.working_y = params["working-y"];
	component.lerp_x = params["lerp-x"];
	component.top_connector = params["top-connector"];
	component.working_x = params["working-x"];
	component.bottom_connector = params["bottom-connector"];
	component.k = params["k"];
	component.prev_x = params["prev-x"];
};

Kiwi.Plugins.Maptile.Component.prototype.executeMessageRecieved = function() {

	if ( Math.random() <= 0.25 ){ 
		return;
	}

	this.randomVar = this.math_random_int(0, this.road_markers.length / 3);

	var inst = this.state.objects.create( this.state.objects.get( "Collectable" ), this.owner.parent );
	inst.visible = true;

	inst.x = this.owner.x + this.road_markers[ ( this.randomVar * 3 ) ];
	inst.y = this.owner.y + this.road_markers[ ( this.randomVar * 3 ) + 1]; 
	var width = this.road_markers[ ( this.randomVar * 3 ) + 2 ] - inst.width;

	inst.x = inst.x + this.math_random_int( -width, width );
	this.messageEachGameObjectWithValue( this.state, 'COLLECTABLE-INSTANCE', inst);	

};

Kiwi.Plugins.Maptile.Component.prototype.onCreate = function() {
	this.owner.addTag('map-tile');
	this.createInitVariables();
	this.arcadephysics.immovable = true;
	this.arcadephysics.enabled = false;
};

Kiwi.Plugins.Maptile.Component.prototype.executeMessageRecieved2 = function() {
	this.messageEachGameObjectWithValue( this.state, 'TILE-INSTANCE',this.owner);
};

Kiwi.Plugins.Maptile.Component.prototype.executeMessageRecieved3 = function() {
	this.checkInstanceWithinBounds(this.instance, true);
};

Kiwi.Plugins.Maptile.Component.prototype.onMessageRecieved = function( name, message ) {
	switch( message ) {
		case 'createCollectable': this.executeMessageRecieved(); break;
		case 'REQUEST-TILE-INSTANCES': this.executeMessageRecieved2(); break;
		case 'CHECK-ON-ROAD':	this.instance = this.owner.properties.get("_messaging-value_");	this.executeMessageRecieved3();break;
	}
};

Kiwi.Plugins.Maptile.Component.prototype.math_random_int = function(a, b) {
	if (a > b) {
		// Swap a and b to ensure a is smaller.
		var c = a;
		a = b;
		b = c;
	}
	return Math.floor(Math.random() * (b - a + 1) + a);
}

Kiwi.Plugins.Maptile.Component.prototype.stringToArray = function(road_markers_string) {
	this.road_markers_string = road_markers_string;
	this.road_markers = [];
	for (var i_index in this.road_markers_string.split(':')) {
		this.i = this.road_markers_string.split(':')[i_index];
		for (var j_index in this.i.split(',')) {
			this.j = this.i.split(',')[j_index];
			this.road_markers.push( parseInt( this.j ) );
		}
		this.road_markers.push( 100 );
	}
}

Kiwi.Plugins.Maptile.Component.prototype.createInitVariables = function() {
	if (this.road_markers_string != null && this.road_markers_string.length > 0) {
		this.stringToArray(this.road_markers_string);
	} else {
		this.stringToArray('90,0:90,50:88,100:88,150:95,200:108,250:137,300:169,350:198,400:209,450:205,500:203,550:205,600:224,650:247,700:269,750:273,800:273,850:270,900:270,950:0,0');
	}
	if (this.road_markers == null || !this.road_markers.length) {
		this.road_markers = [this.game.stage.width / 2, 0, 100];
	}
	if (this.top_connector == null || this.top_connector == 0) {
		this.top_connector = 2;
	}
	if (this.bottom_connector == null || this.bottom_connector == 0) {
		this.bottom_connector = 2;
	}
	this.owner.addTag((String('bottom-connector-') + String(this.bottom_connector)));
	this.owner.addTag((String('top-connector-') + String(this.top_connector)));
}

Kiwi.Plugins.Maptile.Component.prototype.messageEachGameObjectWithValue = function( group, message, value ) {
	group.forEach( this, function( child ) {
		child.properties.set( "_messaging-value_", value );
		child.properties.set( "_messaging_", message );
		if( child.childType() == Kiwi.Group ) {
			this.messageEachGameObjectWithValue( child, message, value );
		}
	});
	return;
}

Kiwi.Plugins.Maptile.Component.prototype.checkInstanceWithinBounds = function(instance, useCenter) {
	this.instance = instance;
	this.useCenter = useCenter;
	if (this.useCenter == null) {
		this.useCenter = true;
	}
	this.instance_x = this.instance.x;
	this.instance_y = this.instance.y;
	if (this.useCenter) {
		this.instance_x = this.instance_x + this.instance.width / 2;
		this.instance_y = this.instance_y + this.instance.height / 2;
	}
	this.k = 0;
	this.prev_x = this.owner.x + this.road_markers[0];
	this.prev_y = this.owner.y + this.road_markers[1];
	var k_end = this.road_markers.length;
	var k_inc = 3;
	if (1 > k_end) {
		k_inc = -k_inc;
	}
	for (k = 1; k_inc >= 0 ? k <= k_end : k >= k_end; k += k_inc) {
		this.k = k
		this.working_x = this.owner.x + this.road_markers[this.k - 1];
		this.working_y = this.owner.y + this.road_markers[(this.k + 1) - 1];
		if (this.instance_y <= this.working_y) {
			this.checkInstanceOnRoad();
			break;
		}
		this.prev_x = this.working_x;
		this.prev_y = this.working_y;
	}
}

Kiwi.Plugins.Maptile.Component.prototype.checkInstanceOnRoad = function() {
	if (this.prev_y == this.working_y) {
		this.lerp_y = this.working_y;
		this.lerp_x = this.working_x;
	} else {
		this.lerp_y = (this.instance_y - this.prev_y) / (this.working_y - this.prev_y);
		this.lerp_x = this.lerp_y * this.working_x + (1 - this.lerp_y) * this.prev_x;
		this.lerp_y = this.lerp_y * this.working_y + (1 - this.lerp_y) * this.prev_y;
	}
	if (Math.abs(this.instance_x - this.lerp_x) > this.road_markers[(this.k + 2) - 1]) {
		this.instance.properties.set("_messaging_", 'OFF-ROAD');
	}
}

Kiwi.Plugins.Maptile.Component.prototype.renderDebug = function() {

	if( !this.game.debugMode ) {
		return;
	}

	this.game.stage.dctx.beginPath();

	for( var j = 1; j <= 2; j++) {

		for( var i = 0; i < this.road_markers.length; i += 3) {

			var x = this.road_markers[i];
			var y = this.road_markers[i+1];
			var width = this.road_markers[i+2];

			if( j == 1 ) {
				width = -width;
			}
			
			if( i === 0) {
				this.game.stage.dctx.moveTo(x + this.owner.x + width, y + this.owner.y);
			} else {
				this.game.stage.dctx.lineTo(x + this.owner.x + width, y + this.owner.y);
			}

		}

		this.game.stage.dctx.strokeStyle = '#ffffff';
		this.game.stage.dctx.stroke();

	}

};

Kiwi.Plugins.Maptile.Component.prototype.update = function() {
	Kiwi.Component.prototype.update.call( this );
	this.renderDebug();
};

Kiwi.Plugins.Maptile.Component.prototype.destroy = function(){


	Kiwi.Component.prototype.destroy.call( this );
	this.state = null;
	this.box = null;
};

Kiwi.Plugins.Maptile.kiwifrootPlugins.push({
	type: Kiwifroot.Plugins.COMPONENT,
	name: 'Maptile',
	namespace: Kiwi.Plugins.Maptile.Component
});
