Kiwi.Plugins.KiwifrootLeaderboardPortrait = {
	name:'KiwifrootLeaderboardPortrait',
	version:'0.5.0',
	minimumKiwiVersion:'1.1.1',
	pluginDependencies:  [
		{
			name: "SocialConnect",
			minimumVersion: "0.8.0"
		},
		{
			name: "GamefrootLeaderboard",
			minimumVersion: "0.1.0"
		}
	],
	kiwifrootPlugins:[]
};

Kiwi.PluginManager.register( Kiwi.Plugins.KiwifrootLeaderboardPortrait );

/**
* Namespace where the developer can enter their facebook app id.
*  
* @property FB_APPID 
* @type string
* @static
*/
Kiwi.Plugins.KiwifrootLeaderboardPortrait.FB_APPID = '';

/**
* Namespace where the developer can enter their gamefroot id.
* Default is valid. 
* 
* @property GF_ID
* @type string
* @static
*/
Kiwi.Plugins.KiwifrootLeaderboardPortrait.GF_ID = '4cc19c277e0cd2939b5f3800196b2b05';

/**
* Used to indicate whether or not the needed init methods 
* (social connect/gf leaderboards) have been executed or not.
*
* @property INITALIZED
* @type boolean
* @static
*/
Kiwi.Plugins.KiwifrootLeaderboardPortrait.INITIALIZED = false;


/**
* Initially executed when the game boots. 
* Executes the init methods of the dependant plugins.
*
* @method create
* @private
*/
Kiwi.Plugins.KiwifrootLeaderboardPortrait.create = function( game ) {
	
	if( !Kiwi.Plugins.KiwifrootLeaderboardPortrait.INITIALIZED ) {	

		if( Kiwi.Plugins.KiwifrootLeaderboardPortrait.FB_APPID && Kiwi.Plugins.KiwifrootLeaderboardPortrait.GF_ID ) {
		
			game.social.facebook.init( {
				appId: Kiwi.Plugins.KiwifrootLeaderboardPortrait.FB_APPID
			} );

			Kiwi.Plugins.KiwifrootLeaderboardPortrait.INITIALIZED = true;

			game.leaderboard.init( Kiwi.Plugins.KiwifrootLeaderboardPortrait.GF_ID, false );

		} else {
			Kiwi.Log.error('Cannot start the Leaderboards as the FB_APPID or GF_ID is missing.');
		}
	
	}

};

/**
* System Class which contains the leaderboard functionality.
* You can add this to a Kiwifroot Level by adding the name of this plugin
* 'KiwifrootLeaderboardPortrait' to the levels json as a new 'system'.
* 
* @class System
*/
Kiwi.Plugins.KiwifrootLeaderboardPortrait.System = function( state ) {

	Kiwi.Component.call(this, state, 'KiwifrootLeaderboardPortrait');

	this.state = state;

	this.score = 0;
};

//Extends a component
Kiwi.extend(Kiwi.Plugins.KiwifrootLeaderboardPortrait.System, Kiwi.Component);

//Add the system to kiwifroot 
Kiwi.Plugins.KiwifrootLeaderboardPortrait.kiwifrootPlugins.push( {
	type: Kiwifroot.Plugins.SYSTEM,
	system: Kiwi.Plugins.KiwifrootLeaderboardPortrait.System
} );

/**
* Executed by the level when this System has been 'chosen' to be used.
* 
* @method configure
* @private
*/
Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype.configure = function( params ) {

	this._active = true;
	this.msg = false;
	this.score = 0;
	this.playAgainCallback = null;
	this.context = null;
	this.pointer = null;
	this._groupInitY = 130;
	this._groupInitX = 30;
	this._textScoreSpacing = 115;

	this._font = 'sans-serif';
	this._fontSize = 40;
	this._fontColor = '#ffffff';
	this._scoreSpacing = 43;

	this._scores = [];

	//Scrolling Variables...
	this._init = 0;
	this._max = 0;
	this.currentlyActive = false;

	this.createGameObjects();
	//this.updateLeaderboard();
	//this.hideLeaderboard();

	params = params || {};
};

/**
* Displays the leaderboard
* 
* @method showLeaderboard
* @param [params] {Object}
* 	@param [params.score=0] {Number} The users score.
* 	@param [params.playAgainCallback=null] {Function} The function to executed when the user pressed 'play again'
* 	@param [params.context] {Any} The context that the 'playAgainCallback' should be executed in.
* 	@param [params.font='sans-serif'] {String} The font which should be used for the leaderboard scores.
* 	@param [params.fontSize=78] {Number} The size of the font in pixels
*	@param [params.fontColor='#ffffff'] {String} The color of the font
* 	@param [params.scoreSpacing] {String} The spacing between score lines.
*	@param [params.textScoreSpacing=115] {Number} The amount of spacing between the score and username
* @private
*/
Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype.showLeaderboard = function( params ) {

	if( !this._active ) {
		Kiwi.Log.error("You need to add this system to the level before you can execute 'showLeaderboard'.");
		return;
	}

	params = params || {};

	this.currentlyActive = true;
	this.masterGroup.visible = true;
	this.submitScore.visible = false;
	this.state.addChild( this.masterGroup );//Appear at the front

	if( params.score && params.score > 0 ) {
		this.score = params.score;
		this.submitScore.visible = true;
	} else {
		this.score = 0;
	}

	if( params.playAgainCallback ) {
		this.playAgainCallback = params.playAgainCallback;
		this.context = params.context;
	} else {
		this.playAgainCallback = null;
		this.context = null;
	}

	if( params.font ) {
		this._font = params.font;
	}

	if( params.fontSize ) {
		this._fontSize = params.fontSize;
	}

	if( params.fontColor ) {
		this._fontColor = params.fontColor;
	}

	if( typeof params.scoreSpacing !== "undefined" ) {
		this._scoreSpacing = params.scoreSpacing;
	} else {
		this._scoreSpacing = this._fontSize + 12;
	}

	if( typeof params.textScoreSpacing !== "undefined" ) {
		this._textScoreSpacing = params.textScoreSpacing;
	}

	this.updateLeaderboard();

	//Add Inputs
	this.game.input.onUp.add( this.onRelease, this, 10000 ); 
	this.game.input.onDown.add( this.onPress, this, 10000 ); 

};

/**
* 
* @method hideLeaderboard
* @private
*/
Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype.hideLeaderboard = function() {
	this.game.input.onUp.remove( this.onRelease, this );
	this.game.input.onDown.remove( this.onPress, this );
	this.currentlyActive = false;
	this.masterGroup.visible = false;
};

Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype.updateLeaderboard = function() {
	
	this.msg = 'The leaderboards could not be fetched. Please make sure you have an internet connection.';
	this.game.leaderboard.get( {
		context: this,
		onError: this.cannotProgress,
		onSuccess: this.renderLeaderboard
	} );

}


Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype.onRelease = function(x, y, td, tu, d, pointer) {

	this.game.input.onUp.halt();

	if( !this.pointer ) {
		return;
	}

	if( Math.abs(this.pointer.startPoint.x - this.pointer.point.x) < 100 && d < 300) {

		if( this.score > 0 && this.submitScore.visible ) {

			if( this.submitScore.box.hitbox.contains( x, y ) ) {
				this._login();
				return;
			}

		}

		if( this.playAgain.box.hitbox.contains( x, y ) ) {
			if( this.playAgainCallback ) {
				this.playAgainCallback.call( this.context );
			} else {
				this.game.info.switchLevelByNumber(0);
			}
			return;
		}

		if( this.closeBtn.box.hitbox.contains( x, y ) ) {
			this.hideLeaderboard();
			return;
		}

	}

	if( this.pointer && this.pointer.id === pointer.id ) {
		this.pointer = null;
	}

};

Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype.onPress = function(x, y, td, tu, d, pointer) {

	this.game.input.onDown.halt();

	if( !this.pointer ) {
		this.pointer = pointer;
		this._init = this.group.y;
	}

};

Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype.update = function() {

	if( this.pointer ) {

		this.group.y = this._init + this.pointer.startPoint.y - this.pointer.point.y;

		if( this.group.y > this._groupInitY ) {
			this.group.y = this._groupInitY;
		} 

		if( this.group.y < -this._max ) {
			this.group.y = -this._max;
		}

		var scrollPercent = (this.group.y - this._groupInitY) / (this._max + this._groupInitY);
		this.scrollBtn.y = this.scrollBar.y + -scrollPercent * (this.scrollBar.height - this.scrollBtn.height);

	}

};

Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype._login = function() {

	this.game.social.gamefroot.loginWithFB( { 
		callback: function(loggedIn) {
			if( loggedIn ) {
				this._post();
			}
		},
		context: this
	} );

};


Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype._post = function() {

	if( this.score > 0 ) {
		//Get the users information from facebook
		this.game.social.facebook.me( {
			autoLoadPicture: false,
			context: this,
			callback: function( resp ) {

				//Submit the score to the leaderboard
				this.msg = "Your score could not be posted.";
				this.game.leaderboard.post( {
					user: resp['first_name'] + ' ' + resp['last_name'],
					score: this.score,
					context: this,
					onError: this.cannotProgress,
					onSuccess: this._resetScore
				});

			}
		});
	}

};

Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype._resetScore = function() {
	this.score = 0;
	this.msg = false;
	this.postScore.exists = false;
	this.updateLeaderboard();
};

Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype.renderLeaderboard = function( scores ) {

	this.msg = false;

	var i = this._scores.length;
	while ( i-- ) {
		this._scores[ i ].exists = false;
	}

	this._scores = [];
	this.group.y = this._groupInitY;
	
	var y = 0, data, user, score, bg;

	for( var i = 0; i < scores.length; i++ ) {
		data = scores[ i ];

		if( data.user == "undefined undefined" ) {
			data.user = 'The Wizard';
		}

		user = this.state.objects.create({
			"type": 'text', 
			"x": 0 + this._textScoreSpacing,
			"y": y,
			"text": data.user,
			"textAlign": "left",
			"fontSize": this._fontSize,
			"fontFamily": this._font,
			"color": this._fontColor

		}, this.group, false );

		score = this.state.objects.create({
			"type": 'text', 
			"x": 0,
			"y": y,
			"text": data.score,
			"textAlign": "left",
			"fontSize": this._fontSize,
			"fontFamily": this._font,
			"color": this._fontColor
		}, this.group, false );

		this._scores.push( user );
		this._scores.push( score );

		y += this._scoreSpacing;

	}

	this._max = y;
	this._max +=  this.playAgain.height + this.submitScore.height - this.game.stage.height;
	if( this._max < 0 ) this._max = 0;

};


Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype.destroy = function(){
	this.game.input.onUp.remove( this.onRelease, this );
	this.game.input.onDown.remove( this.onPress, this );

	Kiwi.Component.prototype.destroy.call( this );

	this.state = null;
	this.pointer = null;
	this.group = null;
};

// 
Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype.cannotProgress = function() {
	if( this.msg ) {
		alert( this.msg );
		this.msg = false;
	} else {
		alert( 'Sorry, the request could not be completed.');
	}
};

// Private
Kiwi.Plugins.KiwifrootLeaderboardPortrait.System.prototype.createGameObjects = function() {

	//MasterGroup
	this.masterGroup = this.state.objects.create( {
		'type': "group",
		'x': 0,
		'y': 0,
		'gameObjects': []
	}, this.state, false);

	this.background = this.state.objects.create( {
		"type": "sprite",
		"x": 0, 
		"y": 0,
		"imageId": 10000
	}, this.masterGroup, false );

	//Group where the scores will be saved
	this.group = this.state.objects.create( {
		"type": "group",
		"x": this._groupInitX,
		"y": this._groupInitY,
		"gameObjects": []
	}, this.masterGroup, false );

	//Title
	this.title = this.state.objects.create( {
		'type': "sprite",
		"imageId": 10001,
		"x": 0,
		"y": 0
	}, this.masterGroup, false );

	//CloseBtn
	this.closeBtn = this.state.objects.create( {
		'type': "sprite",
		"imageId": 10002,
		"x": this.title.x + this.title.width,
		"y": 0
	}, this.masterGroup, false );


	//Scroll Bar 
	this.scrollBar = this.state.objects.create( {
		"type": "sprite",
		"imageId": 10003,
		"x": 0,
		"y": 128
	}, this.masterGroup, false );
	this.scrollBar.x = this.game.stage.width - this.scrollBar.width - 14;

	//Scroll Btn
	this.scrollBtn = this.state.objects.create( {
		"type": "sprite",
		"imageId": 10004,
		"x": this.scrollBar.x,
		"y": this.scrollBar.y
	}, this.masterGroup, false );

	this.playAgain = this.state.objects.create( {
		"type": "sprite",
		"imageId": 10005,
		"x": 0,
		"y": 0
	}, this.masterGroup, false );
	this.playAgain.y = this.game.stage.height - this.playAgain.height;

	this.submitScore = this.state.objects.create( {
		"type": "sprite",
		"imageId": 10006,
		"x": 0,
		"y": 0
	}, this.masterGroup, false );
	this.submitScore.y = this.playAgain.y - this.submitScore.height;
	this.submitScore.visible = false;

};